/*
 * Copyright (c) 2020, 2021, Oracle and/or its affiliates.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2.0, as
 * published by the Free Software Foundation.
 *
 * This program is also distributed with certain software (including
 * but not limited to OpenSSL) that is licensed under separate terms,
 * as designated in a particular file or component or in included license
 * documentation.  The authors of MySQL hereby grant you an
 * additional permission to link the program and your derivative works
 * with the separately licensed software that they have included with
 * MySQL.
 *
 * Without limiting anything contained in the foregoing, this file,
 * which is part of MySQL Connector/Node.js, is also subject to the
 * Universal FOSS Exception, version 1.0, a copy of which can be found at
 * http://oss.oracle.com/licenses/universal-foss-exception.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License, version 2.0, for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */

'use strict';

/* eslint-env node, mocha */

const config = require('../../../config');
const expect = require('chai').expect;
const fixtures = require('../../../fixtures');
const mysqlx = require('../../../../');

describe('autonomous prepared statements for tables without server support', () => {
    let schema, session, table;

    // server container defined in docker.compose.yml
    const baseConfig = { host: 'mysql-8.0.3', socket: undefined };

    beforeEach('create default schema', () => {
        return fixtures.createSchema(config.schema, baseConfig);
    });

    beforeEach('create session using default schema', () => {
        return mysqlx.getSession(Object.assign({}, config, baseConfig))
            .then(s => {
                session = s;
            });
    });

    beforeEach('load default schema', () => {
        schema = session.getSchema(config.schema);
    });

    beforeEach('create table', () => {
        return session.sql('CREATE TABLE test (_id VARBINARY(32), name VARCHAR(4))')
            .execute();
    });

    beforeEach('get table', () => {
        table = schema.getTable('test');
    });

    beforeEach('add fixtures', () => {
        return table.insert('_id', 'name')
            .values('1', 'foo')
            .values('2', 'bar')
            .values('3', 'baz')
            .execute();
    });

    afterEach('drop default schema', () => {
        return session.dropSchema(config.schema);
    });

    afterEach('close session', () => {
        return session.close();
    });

    context('select', () => {
        it('falls back to the regular execution mode', () => {
            // BUG#30162858 _id is a VARBINARY which should map to a Node.js Buffer.
            // eslint-disable-next-line node/no-deprecated-api
            const expected = [[new Buffer('1')], [new Buffer('2')], [new Buffer('3')]];
            const actual = [];

            const op = table.select('_id').where('name = :name');
            const names = ['foo', 'bar', 'baz'];

            return Promise.all(names.map(name => op.bind('name', name).execute(row => actual.push(row))))
                .then(() => expect(actual).to.deep.equal(expected));
        });
    });

    context('update', () => {
        it('falls back to the regular execution mode', () => {
            // BUG#30162858 _id is a VARBINARY which should map to a Node.js Buffer.
            // eslint-disable-next-line node/no-deprecated-api
            const expected = [[new Buffer('1'), 'qux'], [new Buffer('2'), 'qux'], [new Buffer('3'), 'qux']];
            const actual = [];

            const op = table.update().where('name = :name').set('name', 'qux');
            const names = ['foo', 'bar', 'baz'];

            return Promise.all(names.map(name => op.bind('name', name).execute()))
                .then(() => table.select().orderBy('_id').execute(row => actual.push(row)))
                .then(() => expect(actual).to.deep.equal(expected));
        });
    });

    context('delete', () => {
        it('falls back to the regular execution mode', () => {
            const actual = [];

            const op = table.delete().where('name = :name');
            const names = ['foo', 'bar', 'baz'];

            return Promise.all(names.map(name => op.bind('name', name).execute()))
                .then(() => table.select().execute(doc => actual.push(doc)))
                .then(() => expect(actual).to.be.empty);
        });
    });
});
